/****************************************************************************

 ■PhotoFrame
    表示クラス
        Copyright(C) 2018 Denshi Usagi

****************************************************************************/

#include "Display.h"
#include "Weather.h"
#include "main.h"
#include "SDL2/SDL.h"
#include "SDL_image.h"
#include "SDL_ttf.h"
#include "Kosa.h"
#include <math.h>
#include <time.h>

// M+FONTS(http://mplus-fonts.osdn.jp/about.html)
//#define DISP_FONT	"/usr/share/fonts/truetype/mfont/mplus-2p-thin.ttf"
//#define DISP_FONT	"/usr/share/fonts/truetype/mfont/mplus-2p-light.ttf"
#define DISP_FONT_R	"/usr/share/fonts/truetype/mfont/mplus-2p-regular.ttf"
//#define DISP_FONT_M	"/usr/share/fonts/truetype/mfont/mplus-2p-medium.ttf"
#define DISP_FONT_B	"/usr/share/fonts/truetype/mfont/mplus-2p-bold.ttf"
//#define DISP_FONT	"/usr/share/fonts/truetype/mfont/mplus-2p-heavy.ttf"
//#define DISP_FONT	"/usr/share/fonts/truetype/mfont/mplus-2p-black.ttf"

#define IDX_OLD			0
#define IDX_NEW			1

#define WDT_WEEKDAY		0
#define WDT_WEEKSAT		1
#define WDT_WEEKSUN		2
#define WDT_HOLIDAY		3

static SDL_Color s_DayCols[] = {
	{ 0x30, 0x30, 0x30, 0xFF }, // 平日
	{ 0x30, 0x30, 0xB0, 0xFF }, // 土曜日
	{ 0xB0, 0x30, 0x30, 0xFF }, // 日曜日,振替休日
	{ 0x30, 0xA0, 0x30, 0xFF }  // 祝日
};

static PCSTR s_pWeek[7] = { "日", "月", "火", "水", "木", "金", "土", };

// 時間背景
#define MK(x) { (x >> 16) & 0xFF, (x >> 8) & 0xFF, (x >> 0) & 0xFF, 0xFF }
static SDL_Color HourCols[24] = {
	MK(0x5967FF), MK(0x7285F7), MK(0x91ABEE), MK(0xABCAE6), MK(0xAECDE5), MK(0xB1D6DD), MK(0xB5E5CF), MK(0xB9F3C1),
	MK(0xBFFFB7), MK(0xCCFFB7), MK(0xDAFFB7), MK(0xE5FFB7), MK(0xF3FFB7), MK(0xFDFBB5), MK(0xFEE6B0), MK(0xFED1A9),
	MK(0xFFBCA2), MK(0xFFA59B), MK(0xE998A7), MK(0xC78CBC), MK(0xA480D1), MK(0x8275E6), MK(0x5F69FB), MK(0x5967FF), 
};

//---------------------------------------------------------------------------

// ●コンストラクタ
CDisplay::CDisplay()
{
	m_pWnd = NULL;
	m_pSurface = NULL;

	memset(m_Parts, 0, sizeof(m_Parts));
	m_Idxes[IDX_OLD] = 0;
	m_Idxes[IDX_NEW] = 1;
}

// ●表示器の作成
bool CDisplay::Create(void)
{
	// メインウィンドウ作成 ※SDL_WINDOW_FULLSCREENを使うとスタートアップ起動で固まりやすい
	m_pWnd = SDL_CreateWindow("PhotoFrame", 0 ,0, DISP_W, DISP_H,
		SDL_WINDOW_SHOWN | SDL_WINDOW_BORDERLESS /*| SDL_WINDOW_FULLSCREEN*/ );
	if (m_pWnd == NULL)
		return false;

	// ウィンドウサーフェイスの取得
	m_pSurface = SDL_GetWindowSurface(m_pWnd);
	if (m_pSurface == NULL)
		return false;

	SDL_FillRect(m_pSurface, NULL, SDL_MapRGB(m_pSurface->format, 0, 0, 0));
	SDL_UpdateWindowSurface(m_pWnd);

	// 初期ブロックサーフェースの作成
	for(int i = 0; i < PART_X; i++)
	{
		for(int j = 0; j < PART_Y; j++)
		{
			SDL_Surface* p = SDL_CreateRGBSurfaceWithFormat(0,
				PART_L, PART_L, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
			if (p == NULL)
			{
				MessageBox("CDisplay", SDL_GetError());
				continue;
			}

			SDL_FillRect(p, NULL, SDL_MapRGB(p->format, 0, 0, 0));
			m_Parts[m_Idxes[IDX_NEW]][i][j] = p;
		}
	}

	// マウスカーソルを消す
	SDL_ShowCursor(SDL_DISABLE);

	return true;
}

// ●表示器の破棄
void CDisplay::Destroy(void)
{
	// マウスカーソル復活
	SDL_ShowCursor(SDL_ENABLE);

	// ブロックサーフェイスの破棄
	for(int i = 0; i < 2; i++)
	{
		for(int j = 0; j < PART_X; j++)
		{
			for(int k = 0; k < PART_Y; k++)
			{
				SDL_Surface** p = &m_Parts[i][j][k];
				if (*p != NULL)
					SDL_FreeSurface(*p);
			}
		}
	}

	// メインウィンドウの破棄
	if (m_pWnd != NULL)
	{
		SDL_DestroyWindow(m_pWnd);
		m_pWnd = NULL;
	}
}

// --------------------------------------------------------------------------------------
// ●画像の表示
void CDisplay::DisplayImage(PCSTR pFile)
{
	// 画像を読み込む
	SDL_Surface* pImage = IMG_Load(pFile);
	if (pImage == NULL)
	{
		MessageBox("CDisplay", IMG_GetError());
		return;
	}

	// ウィンドウに表示
	Display(pImage);
}

// --------------------------------------------------------------------------------------
// ●カレンダーの表示
void CDisplay::DisplayCalender(double dTempture)
{
	struct tm Tm, Tp;
	time_t tCurrent = time(NULL);
	Tm = Tp = *localtime(&tCurrent);
	Tm.tm_year += 1900;
	Tm.tm_mon++;

	SDL_Color cDarkGray = { 0x20, 0x20, 0x20, 0xFF };
	char buf[64];

	// 描画サーフェイス
	SDL_Surface* pDraw = SDL_CreateRGBSurfaceWithFormat(0,
		m_pSurface->h, m_pSurface->h, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
	if (pDraw == NULL)
	{
		MessageBox("CDisplay", SDL_GetError());
		return;
	}

	// 背景イメージ
	snprintf(buf, sizeof(buf), "calender/image%02u.jpg", Tm.tm_mon);
	SDL_Surface* pImage = IMG_Load(buf);
	if (pImage != NULL)
	{
		SDL_BlitSurface(pImage, NULL, pDraw, NULL);
		SDL_FreeSurface(pImage);
	}
	else {
		MessageBox("CDisplay", IMG_GetError());
	}

	// 西暦,和暦
	snprintf(buf, sizeof(buf), "  %u (R.%u)", Tm.tm_year, (Tm.tm_year % 100) - 18);
	DrawText(pDraw, 700, 5, 220, cDarkGray, buf);

	// 月
	snprintf(buf, sizeof(buf), "%u", Tm.tm_mon);
	DrawText(pDraw, 40 + ((Tm.tm_mon < 10)? 200 : 0), 140, 350, cDarkGray, buf);

	DrawText(pDraw, 320, 290, 390, cDarkGray, "／", DISP_FONT_R);

	// 日
	SDL_Color col = s_DayCols[GetWeekdayType(&Tp)];
	snprintf(buf, sizeof(buf), "%u", Tm.tm_mday);
	DrawText(pDraw, 530 + ((Tm.tm_mday < 10)? 230 : 0), 300, 860, col, buf);

	// 曜
	snprintf(buf, sizeof(buf), "(%s)", s_pWeek[Tm.tm_wday]);
	DrawText(pDraw, 1520, 1120, 270, col, buf);

	// システム温度
	snprintf(buf, sizeof(buf), "System tempture: %4.1f℃", dTempture);
	DrawText(pDraw, 10, 1520, 10, col, buf);
	
	// 左に回転したサーフェイスを得る
	pDraw = LeftRotation(pDraw);
	if (pDraw != NULL)
	{
		// ウィンドウに表示
		Display(pDraw);
	}
}

// ●曜日タイプの取得
int CDisplay::GetWeekdayType(struct tm* p)
{
	// 日曜日
	if (p->tm_wday == 0) 
		return WDT_WEEKSUN;
	// 祝日
	if (IsHolyday(p))
		return WDT_HOLIDAY;
	// 振替休日
	if (IsFurikae(p))
		return WDT_HOLIDAY;
	// 土曜日
	if (p->tm_wday == 6) 
		return WDT_WEEKSAT;
	// 平日
	return WDT_WEEKDAY;
}

// ●祝日かどうかを調べる
bool CDisplay::IsHolyday(struct tm* p)
{
	//【春分日/秋分日】
	// 2010年 3月21日(日)  9月23日(木)
	// 2011年 3月21日(月)  9月23日(金)
	// 2012年 3月20日(火)  9月22日(土)
	// 2013年 3月20日(水)  9月23日(月)
	// 2014年 3月21日(金)  9月23日(火)
	// 2015年 3月21日(土)  9月23日(水)
	// 2016年 3月20日(日)  9月22日(木)
	// 2017年 3月20日(月)  9月23日(土)
	// 2018年 3月21日(水)  9月23日(日)
	// 2019年 3月21日(木)  9月23日(月)
	// 2020年 3月20日(金)  9月22日(火)
	// 2021年 3月20日(土)  9月23日(木)
	// 2022年 3月21日(月)  9月23日(金)
	// 2023年 3月21日(火)  9月23日(土)
	// 2024年 3月20日(水)  9月22日(日)
	// 2025年 3月20日(木)  9月23日(火)
	// 2026年 3月20日(金)  9月23日(水)
	// 2027年 3月21日(日)  9月23日(木)
	// 2028年 3月20日(月)  9月22日(金)
	// 2029年 3月20日(火)  9月23日(日)
	// 2030年 3月20日(水)  9月23日(月)
	// 2031年 3月21日(金)  9月23日(火)
	// 2032年 3月20日(土)  9月22日(木)
	// 2033年 3月20日(日)  9月23日(金)
	static int Syun[] = {
		21, 21, 20, 20, 21, 21, 20, 20, 21, 21,	20, 20,
		21, 21, 20, 20, 20, 21, 20, 20, 20, 21, 20, 20
	};
	static int Syuu[] = {
		23, 23, 22, 23, 23, 23, 22, 23, 23, 23, 22, 23,
		23, 23, 22, 23, 23, 23, 22, 23, 23, 23, 22, 23
	};
	//【日本の休日】
	switch (p->tm_mon + 1)
	{
	case 1:
		// 元日：1/1
		if (p->tm_mday == 1)
			return true;
		// 成人の日：1月第2月曜
		if (p->tm_wday == 1 && ((p->tm_mday - 1) / 7) == 1)
			return true;
		break;
	case 2:
		// 建国記念の日：2/11
		if (p->tm_mday == 11)
			return true;
		// 天皇誕生日：2/23
		if (p->tm_mday == 23)
			return true;
		break;
	case 3:
		// 春分の日：春分日
		if (p->tm_mday == Syun[p->tm_year - 110])
			return true;
		break;
	case 4:
		// 昭和の日：4/29
		if (p->tm_mday == 29)
			return true;
		break;
	case 5:
		// 憲法記念日	：5/3
		// みどりの日：5/4
		// こどもの日：5/5
		if (3 <= p->tm_mday && p->tm_mday <= 5)
			return true;
		break;
	case 6:
		break;
	case 7:
		// 海の日：7月第3月曜日
		if (p->tm_wday == 1 && ((p->tm_mday - 1) / 7) == 2)
			return true;
		break;
	case 8:
		// 山の日：8/11
		if (p->tm_mday == 11)
			return true;
		break;
	case 9:
		// 敬老の日：9月第3月曜日
		if (p->tm_wday == 1 && ((p->tm_mday - 1) / 7) == 2)
			return true;
		// 秋分の日：秋分日
		if (p->tm_mday == Syuu[p->tm_year - 110])
			return true;
		break;
	case 10:
		// 体育の日：10月第2月曜
		if (p->tm_wday == 1 && ((p->tm_mday - 1) / 7) == 1)
			return true;
		break;
	case 11:
		// 文化の日：11/3
		if (p->tm_mday == 3)
			return true;
		// 勤労感謝の日：11/23
		if (p->tm_mday == 23)
			return true;
		break;
	case 12:
		break;
	}

	return false;
}

// ●振替日かどうかを調べる
bool CDisplay::IsFurikae(struct tm* p)
{
	// 日曜日は振替日でない
	if (p->tm_wday != 0)
	{
		// その週の日曜日が祝日ならば
		time_t tt = mktime(p) - p->tm_wday * 86400;
		if (IsHolyday(gmtime(&tt)))
		{
			// 最も近い非祝日が振替日
			struct tm* ptm;
			do {
				tt += 86400;
				ptm = gmtime(&tt);
			}
			while (IsHolyday(ptm));
			
			if (ptm->tm_wday == p->tm_wday)
				return true;
		}
	}

	return false;
}

// --------------------------------------------------------------------------------------
// ●天気予報の表示
void CDisplay::DisplayWeather(const SWeather* pWeather)
{
	SDL_Color cDarkGray = { 0x30, 0x30, 0x30, 0xFF };
	SDL_Rect rect;
	char buf[64];

	// 描画サーフェイス
	SDL_Surface* pDraw = SDL_CreateRGBSurfaceWithFormat(0,
		m_pSurface->h, m_pSurface->h, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
	if (pDraw == NULL)
	{
		MessageBox("CDisplay", SDL_GetError());
		return;
	}

	// 背景
	rect.x = 0; rect.y= 0; rect.w = m_pSurface->h; rect.h = 15;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 0, 162, 232));

	rect.x = 0; rect.y= 15; rect.w = m_pSurface->h; rect.h = 1070;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 231, 245, 250));

	rect.x = 0; rect.y= 1085; rect.w = m_pSurface->h; rect.h = 15;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 92, 163, 99));
	
	rect.x = 0; rect.y= 1100; rect.w = m_pSurface->h; rect.h = 436;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 236, 253, 232));

	// 1時間ごとの天気
	for(int i = 0; i < 24; i++)
	{
		const SWeatherDetail* pDetail = &pWeather->Detail[i];

		rect.w = 240; rect.h = 68;
		rect.x = (i % 8) * 256 + 8;
		rect.y = (i / 8) * 346 + 46;

		// 時間背景
		SDL_Color sCol = HourCols[pDetail->nHour];
		SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, sCol.r, sCol.g, sCol.b));

		// 時間テキスト
		snprintf(buf, sizeof(buf), "%d:00", pDetail->nHour);
		DrawText(pDraw, &rect, 50, cDarkGray, buf);

		// 天気アイコン
		snprintf(buf, sizeof(buf), "weather/%03u.png", pDetail->nTenki);
		SDL_Surface* pIcon = IMG_Load(buf);
		if (pIcon != NULL)
		{
			rect.w = pIcon->w; rect.h = pIcon->h;
			rect.x = (i % 8) * 256 + (256 - rect.w) / 2;
			rect.y = (i / 8) * 346 + 122;

			SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
			SDL_FreeSurface(pIcon);
		}

		// 降水量
		if (pDetail->nKosui > 0)
		{
			int n = (double)((pDetail->nKosui < 30)? pDetail->nKosui : 30) * 2;
			SDL_Rect temp = rect;
			temp.x += 30;
			temp.y += 240 - n;
			temp.w -= 60;
			temp.h = n;
			SDL_FillRect(pDraw, &temp, SDL_MapRGB(m_pSurface->format, 149, 181, 255));
		}

		// 気温テキスト
		rect.y += 176, rect.h = 76;
		snprintf(buf, sizeof(buf), "%d℃", pDetail->nKion);
		DrawText(pDraw, &rect, 62, cDarkGray, buf);
	}
	
	// 週間天気(8日)
	time_t tCurrent = time(NULL);
	for (int i = 0; i < 8; i++)
	{
		const SWeatherWeekly* Weekly = &pWeather->Weekly[i];

		// 日テキスト
		rect.w = 256; rect.h = 76;
		rect.x = (i % 8) * 256 + (256 - rect.w) / 2;
		rect.y = 1112;

		struct tm Tm = *localtime(&tCurrent);
		if (Weekly->nDay < Tm.tm_mday) Tm.tm_mon++;
		Tm.tm_mday = Weekly->nDay;
		
		time_t tTemp = mktime(&Tm);
		Tm = *localtime(&tTemp);
	
		int wdt = GetWeekdayType(&Tm);
		SDL_Color col = s_DayCols[wdt];
		SDL_Color* pfc = &cDarkGray;

		if (wdt != WDT_WEEKDAY)
		{
			static SDL_Color cWhite = { 0xFF, 0xFF, 0xFF, 0xFF };
			SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, col.r, col.g, col.b));
			pfc = &cWhite;
		}

		snprintf(buf, sizeof(buf), "%d(%s)", Weekly->nDay, s_pWeek[Tm.tm_wday]);
		DrawText(pDraw, &rect, 62, *pfc, buf);

		// 天気アイコン
		snprintf(buf, sizeof(buf), "weather/%03u.png", Weekly->nTenki);
		SDL_Surface* pIcon = IMG_Load(buf);
		if (pIcon != NULL)
		{
			rect.w = pIcon->w; rect.h = pIcon->h;
			rect.x = (i % 8) * 256 + (256 - rect.w) / 2;
			rect.y = 1196;

			SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
			SDL_FreeSurface(pIcon);
		}
		else
		{
			char buf[64];
			snprintf(buf, sizeof(buf), "天気アイコンがない(%s)", buf);
			g_SaveData.WriteStr("LastError", buf);
		}
		
		// 気温テキスト
		rect.y += 176, rect.h = 76;
		snprintf(buf, sizeof(buf), "%d℃", Weekly->nKion);
		DrawText(pDraw, &rect, 62, cDarkGray, buf);

		// 降水確率テキスト
		rect.y += 64;
		snprintf(buf, sizeof(buf), "%d％", Weekly->nKosui);
		DrawText(pDraw, &rect, 62, cDarkGray, buf);
	}

	// 左に回転したサーフェイスを得る
	pDraw = LeftRotation(pDraw);
	if (pDraw != NULL)
	{
		// ウィンドウに表示
		Display(pDraw);
	}
}

// --------------------------------------------------------------------------------------
// ●黄砂予報の表示
void CDisplay::DisplayKosa(const SKosa* pKosa)
{
	SDL_Color cDarkGray = { 0x30, 0x30, 0x30, 0xFF };
	SDL_Rect rect;
	char buf[64];

	// 描画サーフェイス
	SDL_Surface* pDraw = SDL_CreateRGBSurfaceWithFormat(0,
		m_pSurface->h, m_pSurface->h, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
	if (pDraw == NULL)
	{
		MessageBox("CDisplay", SDL_GetError());
		return;
	}

	// 背景
	rect.x = 0; rect.y= 0; rect.w = m_pSurface->h; rect.h = 15;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 0, 162, 232));

	rect.x = 0; rect.y= 15; rect.w = m_pSurface->h; rect.h = 895;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 231, 245, 250));

	rect.x = 0; rect.y= 910; rect.w = m_pSurface->h; rect.h = 15;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 92, 163, 99));
	
	rect.x = 0; rect.y= 925; rect.w = m_pSurface->h; rect.h = 611;
	SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, 236, 253, 232));
	
	rect.x = 42;
	rect.y = 370;
	rect.w = 200;
	rect.h = 64;
	DrawText(pDraw, &rect, 76, cDarkGray, "PM2.5");
	
	rect.y = 640;
	DrawText(pDraw, &rect, 76, cDarkGray, "黄砂");
	
	// 6時間ごとの予報
	time_t tCurrent = time(NULL);
	struct tm Tm = *localtime(&tCurrent);

	// 時刻バー
	int nTop = (Tm.tm_hour / 6) * 6;
	for(int i = 0; i < 4; i++)
	{
		rect.w = 420; rect.h = 68;
		rect.x = 279 + 440 * i;
		rect.y = 124;

		// 時間背景
		SDL_Color sCol = HourCols[nTop + 2];
		SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, sCol.r, sCol.g, sCol.b));

		// 時間テキスト
		snprintf(buf, sizeof(buf), "%u－%u", nTop, nTop + 6);
		DrawText(pDraw, &rect, 50, cDarkGray, buf);
		nTop = (nTop + 6) % 24;
	}
	
	nTop = -1;
	if (pKosa->nDay1 == Tm.tm_mday)
		nTop = Tm.tm_hour / 6;
	else if (pKosa->nDay2 == Tm.tm_mday)
		nTop = Tm.tm_hour / 6 + 4;
	
	if (nTop != -1)
	{
		// PM2.5
		for(int i = 0, j = nTop; i < 4; i++, j++)
		{
			int nValue = pKosa->PM25_Detail[j];
			if (nValue != 0)
			{
				snprintf(buf, sizeof(buf), "kosa/lv%u.png", nValue);
				SDL_Surface* pIcon = IMG_Load(buf);
				if (pIcon != NULL)
				{
					rect.w = pIcon->w; rect.h = pIcon->h;
					rect.x = 289 + 440 * i;
					rect.y = 278;

					SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
					SDL_FreeSurface(pIcon);
				}
			}
		}

		// 黄砂
		for(int i = 0, j = nTop; i < 4; i++, j++)
		{
			int nValue = pKosa->Kosa_Detail[j];
			if (nValue != 0)
			{
				snprintf(buf, sizeof(buf), "kosa/lv%u.png", nValue);
				SDL_Surface* pIcon = IMG_Load(buf);
				if (pIcon != NULL)
				{
					rect.w = pIcon->w; rect.h = pIcon->h;
					rect.x = 289 + 440 * i;
					rect.y = 558;

					SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
					SDL_FreeSurface(pIcon);
				}
			}
		}
	}
	
	// 週間日テキスト
	time_t tTemp = tCurrent;
	for (int i = 0; i < 6; i++)
	{
		rect.w = 320; rect.h = 76;
		rect.x = 340 * i + 14 + (320 - rect.w) / 2;
		rect.y = 962;

		tTemp += 24 * 60 * 60;
		struct tm Tm = *localtime(&tTemp);
	
		int wdt = GetWeekdayType(&Tm);
		SDL_Color col = s_DayCols[wdt];
		SDL_Color* pfc = &cDarkGray;

		if (wdt != WDT_WEEKDAY)
		{
			static SDL_Color cWhite = { 0xFF, 0xFF, 0xFF, 0xFF };
			SDL_FillRect(pDraw, &rect, SDL_MapRGB(m_pSurface->format, col.r, col.g, col.b));
			pfc = &cWhite;
		}

		snprintf(buf, sizeof(buf), "%d(%s)", Tm.tm_mday, s_pWeek[Tm.tm_wday]);
		DrawText(pDraw, &rect, 62, *pfc, buf);
	}

	// 週間 PM2.5
	for(int i = 0, j = 1; i < 6; i++, j++)
	{
		int nValue = pKosa->PM25_Weekly[j];
		if (nValue != 0)
		{
			snprintf(buf, sizeof(buf), "kosa/lv%us.png", nValue);
			SDL_Surface* pIcon = IMG_Load(buf);
			if (pIcon != NULL)
			{
				rect.w = pIcon->w; rect.h = pIcon->h;
				rect.x = 24 + 340 * i;
				rect.y = 1069;

				SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
				SDL_FreeSurface(pIcon);
			}
		}
	}
	
	// 週間 黄砂
	for(int i = 0, j = 1; i < 6; i++, j++)
	{
		int nValue = pKosa->Kosa_Weekly[j];
		if (nValue != 0)
		{
			snprintf(buf, sizeof(buf), "kosa/lv%us.png", nValue);
			SDL_Surface* pIcon = IMG_Load(buf);
			if (pIcon != NULL)
			{
				rect.w = pIcon->w; rect.h = pIcon->h;
				rect.x = 24 + 340 * i;
				rect.y = 1289;

				SDL_BlitSurface(pIcon, NULL, pDraw, &rect);
				SDL_FreeSurface(pIcon);
			}
		}
	}

	// 左に回転したサーフェイスを得る
	pDraw = LeftRotation(pDraw);
	if (pDraw != NULL)
	{
		// ウィンドウに表示
		Display(pDraw);
	}
}

// --------------------------------------------------------------------------------------
// ●終了表示
void CDisplay::Ending(void)
{
	// 覆うサーフェース
	SDL_Surface* pCover = SDL_CreateRGBSurfaceWithFormat(0,
		m_pSurface->w, m_pSurface->h, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
	if (pCover == NULL)
	{
		MessageBox("CDisplay", IMG_GetError());
		return;
	}

	// フェードアウト処理
	SDL_FillRect(pCover, NULL, SDL_MapRGB(m_pSurface->format, 0, 0, 0));
	SDL_SetSurfaceBlendMode(pCover, SDL_BLENDMODE_BLEND);

	double a = 0;
	for (int i = 1; a < 255 ; i++)
	{
		a = pow(i, 1.2);
		if (a >= 90) a = 255;

		SDL_SetSurfaceAlphaMod(pCover, a);
		SDL_BlitSurface(pCover, NULL, m_pSurface, NULL);
		SDL_UpdateWindowSurface(m_pWnd);
	}

	SDL_FreeSurface(pCover);
}

// --------------------------------------------------------------------------------------

// ●左に回転したサーフェイスを得る
SDL_Surface* CDisplay::LeftRotation(SDL_Surface* pSurface)
{
	SDL_Surface* pRoated = NULL;

	// 回転後のサーフェイス
	SDL_Surface* pTemp = SDL_CreateRGBSurfaceWithFormat(0,
		m_pSurface->h, m_pSurface->h, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);

	if (pTemp != NULL)
	{
		SDL_Renderer* pRenderer = SDL_CreateSoftwareRenderer(pTemp);
		if (pRenderer != NULL)
		{
			SDL_Texture* pTexture = SDL_CreateTextureFromSurface(pRenderer, pSurface);
			if (pTexture != NULL)
			{
				if (SDL_RenderCopyEx(pRenderer, pTexture, NULL, NULL, -90, NULL, SDL_FLIP_NONE) == 0)
					pRoated = pTemp;

				SDL_DestroyTexture(pTexture);
			}

			SDL_DestroyRenderer(pRenderer);
		}

		if (pRoated == NULL)
			SDL_FreeSurface(pTemp);
	}

	// 受け取ったサーフェイスは破棄
	SDL_FreeSurface(pSurface);

	if (pRoated == NULL)
		MessageBox("CDisplay", SDL_GetError());

	return pRoated;
}

// ●テキストの描画
void CDisplay::DrawText(SDL_Surface* pDraw, int x, int y, int nSize, SDL_Color& col, PCSTR pText, PCSTR pFontName)
{
	bool bSuccess = false;

	// フォントロード
	TTF_Font* pFont = TTF_OpenFont(pFontName, nSize);
	if (pFont != NULL)
	{
		SDL_Surface* pWork = TTF_RenderUTF8_Blended(pFont, pText, col);
		if (pText != NULL)
		{
			// 目的位置に描画
			SDL_Rect rcDraw;
			rcDraw.x = x;
			rcDraw.y = y;
			if (TTF_SizeUTF8(pFont, pText, &rcDraw.w, &rcDraw.h) == 0)
			{
				SDL_BlitSurface(pWork, NULL, pDraw, &rcDraw);
				bSuccess = true;
			}

			SDL_FreeSurface(pWork);
		}
		TTF_CloseFont(pFont);
	}

	if (!bSuccess)
		MessageBox("CDisplay", TTF_GetError());
}

// ●テキストの描画
void CDisplay::DrawText(SDL_Surface* pDraw, SDL_Rect* pRect, int nSize, SDL_Color& col, PCSTR pText, PCSTR pFontName)
{
	bool bSuccess = false;

	// フォントロード
	TTF_Font* pFont = TTF_OpenFont(pFontName, nSize);
	if (pFont != NULL)
	{
		SDL_Surface* pWork = TTF_RenderUTF8_Blended(pFont, pText, col);
		if (pText != NULL)
		{
			int w, h;
			if (TTF_SizeUTF8(pFont, pText, &w, &h) == 0)
			{
				// センタリング描画
				SDL_Rect rcDraw;
				rcDraw.x = pRect->x + (pRect->w - w) / 2;
				rcDraw.y = pRect->y + (pRect->h - h) / 2;
				rcDraw.w = w;
				rcDraw.h = h;
				SDL_BlitSurface(pWork, NULL, pDraw, &rcDraw);
				bSuccess = true;
			}
			SDL_FreeSurface(pWork);
		}
		TTF_CloseFont(pFont);
	}

	if (!bSuccess)
		MessageBox("CDisplay", TTF_GetError());
}

// ●ウィンドウに表示する
void CDisplay::Display(SDL_Surface* pSurface)
{
	// ブロックサーフェースを作成する
	if (m_Idxes[IDX_OLD] == 0)
	{
		m_Idxes[IDX_OLD] = 1;
		m_Idxes[IDX_NEW] = 0;
	}
	else
	{
		m_Idxes[IDX_OLD] = 0;
		m_Idxes[IDX_NEW] = 1;
	}

	SDL_Rect rcSrc;
	rcSrc.w = PART_L;
	rcSrc.h = PART_L;

	for (int i = 0; i < PART_X; i++)
	{
		rcSrc.x = PART_L * i;
		for (int j = 0; j < PART_Y; j++)
		{
			rcSrc.y = PART_L * j;
			SDL_Surface** pPart = &m_Parts[m_Idxes[IDX_NEW]][i][j];

			// 以前のブロックサーフェースを破棄
			if (*pPart != NULL)
			{
				SDL_FreeSurface(*pPart);
				*pPart = NULL;
			}

			// ブロックサーフェースを作成
			*pPart = SDL_CreateRGBSurfaceWithFormat(0,
				PART_L, PART_L, m_pSurface->format->BitsPerPixel, m_pSurface->format->format);
			if (*pPart == NULL)
			{
				MessageBox("CDisplay", SDL_GetError());
				continue;
			}

			// ブロックサーフェースにソースの一部分をコピー
			if (SDL_BlitSurface(pSurface, &rcSrc, *pPart, NULL) != 0)
				MessageBox("CDisplay", SDL_GetError());
		}
	}

	// 渡されたサーフェースは破棄
	SDL_FreeSurface(pSurface);

	//ウィンドウに表示する
	switch (rand() % 3)
	{
	case 0:  Present1(); break;
	case 1:  Present2(); break;
	case 2:  Present3(); break;
	}
}

// ●表示タイプ1
void CDisplay::Present1(void)
{
	// 分解能設定
	SetupState(40);

	// 変数初期化
	bool tbl[PART_X][PART_Y];
	memset(tbl, 0, sizeof(tbl));

	SDL_Rect rcDst;
	rcDst.w = PART_L;
	rcDst.h = PART_L;

	// 全パネルループ
	for(int i = PART_X * PART_Y; i > 0; i--)
	{
		int n = rand() % i;
		for(int j = 0; j < PART_X; j++)
		{
			for(int k = 0; k < PART_Y; k++)
			{
				if (tbl[j][k] == false)
				{
					if (n == 0)
					{
						// 1ブロック回転処理
						tbl[j][k] = true;

						rcDst.x = PART_L * j;
						rcDst.y = PART_L * k;

						SDL_Rect rcPart;
						rcPart.x = rcDst.x;
						rcPart.w = rcDst.w;

						for(int s = 0; s < m_nStage; s++)
						{
							rcPart.h = m_nFlip[s];
							rcPart.y = rcDst.y + (PART_L - rcPart.h) / 2;

							SDL_Surface* pPart = m_Parts[m_Idxes[s / (m_nStage / 2)]][j][k];
							if (pPart != NULL)
							{
								// 描画
								SDL_FillRect(m_pSurface, &rcDst, SDL_MapRGB(m_pSurface->format, 0, 0, 0));
								SDL_BlitScaled(pPart, NULL, m_pSurface, &rcPart);

								// ウィンドウ表示更新
								SDL_UpdateWindowSurfaceRects(m_pWnd, &rcDst, 1);
								SDL_Delay(0);
							}
						}
						goto NEXT;
					}
					--n;
				}
			}
		}
	NEXT:;
	}
}

// ●表示タイプ2
void CDisplay::Present2(void)
{
	#define COUNT_LAG	50

	// 分解能設定
	SetupState(40);

	// 変数初期化
	SDL_Rect rcDests[PART_X];
	for(int i = 0; i < PART_X; i++)
	{
		rcDests[i].x = PART_L * i; 
		rcDests[i].w = PART_L;
		rcDests[i].h = PART_L;
	}

	// 全パネルループ
	int nCount = 0;
	do {
		// 1ステージループ
		for(int i = 0; i < PART_X; i++)
		{
			SDL_Rect* prcDest = &rcDests[i];

			int c = nCount - i * COUNT_LAG;
			if (0 <= c && c < m_nStage * PART_Y)
			{
				prcDest->y = DISP_H - PART_L - (c / m_nStage) * PART_L;

				SDL_Rect rcPart;
				rcPart.x = prcDest->x;
				rcPart.w = prcDest->w;

				int s = c % m_nStage;

				rcPart.h = m_nFlip[s];
				rcPart.y = prcDest->y + (PART_L - rcPart.h) / 2;

				SDL_Surface* pPart = m_Parts[m_Idxes[s / (m_nStage / 2)]][i][PART_Y - c / m_nStage - 1];
				if (pPart != NULL)
				{
					// 描画
					SDL_FillRect(m_pSurface, prcDest, SDL_MapRGB(m_pSurface->format, 0, 0, 0));
					SDL_BlitScaled(pPart, NULL, m_pSurface, &rcPart);
				}
			}
			else {
				prcDest->y = 0;
			}
		}

		// ウインドウ表示更新
		SDL_UpdateWindowSurfaceRects(m_pWnd, rcDests, PART_X);
		SDL_Delay(0);
	}
	while (++nCount < (m_nStage * PART_Y + (PART_X - 1) * COUNT_LAG));
}

// ●表示タイプ3
void CDisplay::Present3(void)
{
	SDL_Rect rcDst;
	rcDst.w = PART_L;
	rcDst.h = PART_L;
	
	// 全パネルループ
	double a = 0;
	for (int i = 1; a < 255 ; i++)
	{
		a = pow(i, 1.2);
		if (a >= 90) a = 255;
				
		for(int j = 0; j < PART_X; j++)
		{
			for(int k = 0; k < PART_Y; k++)
			{
				rcDst.x = PART_L * j;
				rcDst.y = PART_L * k;

				SDL_Surface* pPart = m_Parts[m_Idxes[IDX_NEW]][j][k];

				SDL_SetSurfaceBlendMode(pPart, SDL_BLENDMODE_BLEND);
				SDL_SetSurfaceAlphaMod(pPart, a);

				SDL_BlitSurface(pPart, NULL, m_pSurface, &rcDst);
				SDL_SetSurfaceBlendMode(pPart, SDL_BLENDMODE_NONE);
			}
		}
		SDL_UpdateWindowSurface(m_pWnd);
	}
}

// ●回転の分解能を設定する
void CDisplay::SetupState(int nStage)
{
	m_nStage = nStage;

	for (int i = 0; i < m_nStage; i++)
	{
		double deg = (180 * (i + 1)) / m_nStage;
		m_nFlip[i] = abs(round(cos((deg * 3.14) / 180) * PART_L));
	}
}
