/****************************************************************************

 ■PhotoFrame
	黄砂情報
        Copyright(C) 2021 Denshi Usagi

****************************************************************************/

#include "Kosa.h"
#include "main.h"
#include <stdlib.h>
#include <string.h>

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include "SDL.h"

//---------------------------------------------------------------------------

// ●コンストラクタ
CKosa::CKosa()
{
	m_bEnable = false;
	memset(&m_Kosa, 0, sizeof(m_Kosa));
	m_Socket = -1;
	m_pCtx = NULL;
	m_pSSL = NULL;
	m_tLast = 0;
	m_pBuf = NULL;
}

// ●初期化
void CKosa::Initialize(void)
{
	// 黄砂取得元の情報
	m_strHost = g_Settings.ReadStr("KosaHost", "");
	m_strQuery = g_Settings.ReadStr("KosaQuery", "");
}

// ●ネットから黄砂情報を取得する
void CKosa::Update(void)
{
	// 取得元が設定されていない場合は黄砂表示不可
	if (m_strHost == "" || m_strQuery == "")
		return;

	time_t tCurrent = time(NULL);
	struct tm tmCur = *localtime(&tCurrent);

	// 取得処理(サーバーアクセス)の間引き
	//m_tLast = 0; // for Debug
	if (m_tLast > 0)
	{
		if (m_bEnable)
		{
			// 正常に取得できている場合は、1時間おきに再取得する
			struct tm tmLast = *localtime(&m_tLast);
			if (tmCur.tm_hour == tmLast.tm_hour)
				return;
			if ((tCurrent - m_tLast) < 10)
				return;
		}
		else
		{
			// 正常に取得できていない場合は、1分おきにリトライする
			// URLの変更やHTMLの仕様が変わった可能性もある
			if ((tCurrent - m_tLast) < (1 * 60))
				return;
		}
	}

	// 接続から黄砂情報取得まで
	char buf[128], tbf[64];
	strftime(tbf, sizeof(tbf), "%Y/%m/%d %H:%M:%S", &tmCur);

	int nErr = Request(m_strQuery.c_str(), m_pBuf, 200 * 1024);
	if (nErr == 0)
	{
		nErr = ParseHtml();
		if (nErr != 0)
		{
			snprintf(buf, sizeof(buf), "黄砂情報の取得に失敗(nErr = %d) %s", nErr, tbf);
			g_SaveData.WriteStr("LastError", buf);
		}
	}
	else
	{
		snprintf(buf, sizeof(buf), "黄砂情報のリクエストに失敗(nErr = %d) %s", nErr, tbf);
		g_SaveData.WriteStr("LastError", buf);
	}
	
	// リクエスト終了
	EndRequest();

	m_tLast = tCurrent;
}

// ●黄砂情報が有効かどうか
bool CKosa::IsEnable(void)
{
	return m_bEnable;
}

// ●現在の黄砂情報を取得する
const SKosa* CKosa::GetKosa(void)
{
	return &m_Kosa;
}

// ●HTTPリクエストを送信してHTMLを受信する
int CKosa::Request(PCSTR pQuery, PSTR& pBuf, int nSize)
{
	// バッファ確保
	pBuf = (char*)malloc(nSize + 1);
	if (pBuf == NULL)
		return 1;

	memset(pBuf, 0 , nSize + 1);

	// ホスト解決
	addrinfo hints;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;
	
	addrinfo* res;
    if (getaddrinfo(m_strHost.c_str(), "https", &hints, &res) != 0)
        return 2;
	
	addrinfo info = *res;
	freeaddrinfo(res);

	// ソケット作成
	m_Socket = socket(info.ai_family, info.ai_socktype, info.ai_protocol);
	if (m_Socket == -1)
		return 3;
	
	// ソケット接続
	if (connect(m_Socket, info.ai_addr, info.ai_addrlen) != 0)
		return 4;
	
	// SSL初期化
    m_pCtx = SSL_CTX_new(SSLv23_client_method());
	if (m_pCtx == NULL)
		return 5;
		
	SSL_CTX_set_mode(m_pCtx, SSL_MODE_AUTO_RETRY);

	if (SSL_CTX_set_min_proto_version(m_pCtx, TLS1_2_VERSION) == 0)
		return 6;

	if (SSL_CTX_set_default_verify_paths(m_pCtx) == 0)
		return 7;

    m_pSSL = SSL_new(m_pCtx);
	if (m_pSSL == NULL)
		return 8;
    
    if (SSL_set_tlsext_host_name(m_pSSL, m_strHost.c_str()) == 0)
		return 9;

	if (SSL_set_fd(m_pSSL, m_Socket) == 0)
		return 10;
			
    // SSL接続
	if (SSL_connect(m_pSSL) != 1)
		return 11;
	
	// ヘッダ送信
	String strRequest;
	strRequest.append("GET ");
	strRequest.append(pQuery);
	strRequest.append(" HTTP/1.1\r\nHost: ");
	strRequest.append(m_strHost.c_str());
	strRequest.append("\r\nConnection: close");
	strRequest.append("\r\n\r\n");
	
	int nSend = SSL_write(m_pSSL, strRequest.c_str(), strRequest.size());
	if (nSend != strRequest.size())
		return 12;

	// レスポンス受信
	int nRecv = SSL_read(m_pSSL, pBuf, 17);
	if (nRecv != 17 || strstr(pBuf, "HTTP/1.1 200 OK\r\n") != pBuf)
		return 13;

	// 受信処理
	PSTR pRecv = pBuf;
	while(nSize > 0)
	{
		SDL_Delay(100);
        
		nRecv = SSL_read(m_pSSL, pRecv, nSize);
		if (nRecv <= 0)
			break;
		
		pRecv += nRecv;
		nSize -= nRecv;
	}
	if (pRecv == pBuf)
		return 14;
	
#if 0 // デバッグ用(受信したHTML文書をカレントディレクトリに吐き出す)
	FILE* fp = fopen("Query-recv.txt", "w+");
	if (fp)
	{
		fputs(pBuf, fp);
		fclose(fp);
	}
#endif

	return 0;
}

// ●HTMLを受信して黄砂情報を取得する
int CKosa::ParseHtml(void)
{
	m_bEnable = false;

	PSTR pGet = m_pBuf + 2 * 1024; // 先頭読み飛ばし

	if (!Search(pGet, "fable"))
		return 1;

	// 黄砂情報取得
	SKosa data;
	memset(&data, 0, sizeof(data));

	// 1日目の日
	if (!GetValue(pGet, "月", "日", data.nDay1))
		return 2;

	// 2日目の日
	if (!GetValue(pGet, "月", "日", data.nDay2))
		return 3;
	
	// 1日目 6-12フィールド
	if (!Search(pGet, "6-12"))
		return 4;
	// 1日目 12-18フィールド
	if (!Search(pGet, "12-18"))
		return 5;
	// 1日目 18-24フィールド
	if (!Search(pGet, "18-24"))
		return 6;
	
	// 2日目 0-6フィールド
	if (!Search(pGet, "0-6"))
		return 7;
	// 2日目 6-12フィールド
	if (!Search(pGet, "6-12"))
		return 8;
	// 2日目 12-18フィールド
	if (!Search(pGet, "12-18"))
		return 9;
	// 2日目 18-24フィールド
	if (!Search(pGet, "18-24"))
		return 10;
	
	// PM2.5
	if (!Search(pGet, "PM2.5"))
		return 11;
	
	for(int i = 1; i < 8; i++)
	{
		if (!GetValue(pGet, "class=\"color", "\">", data.PM25_Detail[i]))
			return 12;
	}
	
	// 黄砂
	if (!Search(pGet, "黄砂"))
		return 13;
	
	for(int i = 1; i < 8; i++)
	{
		if (!GetValue(pGet, "class=\"color", "\">", data.Kosa_Detail[i]))
			return 14;
	}
	
	// 週間予報 PM2.5
	if (!Search(pGet, "PM2.5"))
		return 15;
	
	for(int i = 0; i < 7; i++)
	{
		if (!GetValue(pGet, "class=\"color", "\">", data.PM25_Weekly[i]))
			return 16;
	}
	
	// 週間予報 黄砂
	if (!Search(pGet, "黄砂"))
		return 17;
	
	for(int i = 0; i < 7; i++)
	{
		if (!GetValue(pGet, "class=\"color", "\">", data.Kosa_Weekly[i]))
			return 18;
	}
	
	// 取得成功
	m_Kosa = data;
	m_bEnable = true;

	return 0;
}

// ●データ位置のサーチ
bool CKosa::Search(PSTR& pBuf, PCSTR pKey)
{
	PSTR pTop = strstr(pBuf, pKey);
	if (pTop != NULL)
	{
		pBuf = pTop + strlen(pKey);
		return true;
	}
	return false;
}

// ●データ取得位置の検索
bool CKosa::GetValue(PSTR& pBuf, PCSTR pKey1, PCSTR pKey2, int& var)
{
	PSTR pTop = strstr(pBuf, pKey1);
	if (pTop == NULL)
		return false;

	pTop += strlen(pKey1);
	PSTR pEnd = strstr(pTop, pKey2);
	if (pEnd == NULL)
		return false;

	pBuf = pEnd + strlen(pKey2);
	*pEnd = '\0';

	var = atoi(pTop);
	return true;
}

// ●リクエストを終了する
void CKosa::EndRequest(void)
{
	if (m_pSSL != NULL)
	{
		SSL_shutdown(m_pSSL);
		SSL_free(m_pSSL);
		m_pSSL = NULL;
	}
	if (m_pCtx != NULL)
	{
		SSL_CTX_free(m_pCtx);
		m_pCtx = NULL;
	}
	if (m_Socket != -1)
	{
		close(m_Socket);
		m_Socket = -1;
	}
	if (m_pBuf != NULL)
	{
		free(m_pBuf);
		m_pBuf = NULL;
	}
}
