/****************************************************************************

 ■PhotoFrame
	写真リスト管理
        Copyright(C) 2018 Denshi Usagi

****************************************************************************/

#include "PhotoList.h"
#include "main.h"
#include <dirent.h>
#include <sys/stat.h>

//---------------------------------------------------------------------------

// ●コンストラクタ
CPhotoList::CPhotoList()
{
	m_nCurrent = -1;
}

// ●初期化
bool CPhotoList::Initialize(void)
{
	if (!GetFileList("photo"))
		return false;

	// 最後に表示していたファイルを探す
	String sLast = g_SaveData.ReadStr("LastPhoto", "");
	if (sLast != "")
	{
		for(int i = 0; i < m_List.size(); i++)
		{
			if (m_List[i] == sLast)
			{
				m_nCurrent = i;
				break;
			}
		}
	}

	return true;
}

// ●次の写真を取得する
PCSTR CPhotoList::GetNext(void)
{
	#define KEYNAME	"LastPhoto"

	if (m_List.size() == 0)
	{
		g_SaveData.WriteStr(KEYNAME, "");
		return NULL;
	}

	if (++m_nCurrent >= m_List.size())
		m_nCurrent = 0;

	PCSTR pPhoto = m_List[m_nCurrent].c_str();
	g_SaveData.WriteStr(KEYNAME, pPhoto);

	return pPhoto;
}

// ●ファイルリストを取得する(再帰)
bool CPhotoList::GetFileList(PCSTR pPath)
{
	DIR* pDir = opendir(pPath);
	if (pDir == NULL)
		return false;

	bool bResult = true;

	struct dirent* pEnt = readdir(pDir);
	for (; pEnt != NULL; pEnt = readdir(pDir))
	{
		if (pEnt->d_name[0] == '.')
			continue;

		String sPath = pPath;
		sPath += "/";
		sPath += pEnt->d_name;
		
		struct stat st;
		if (stat(sPath.c_str(), &st) != 0)
		{
			bResult = false;
			break;
		}

		if (S_ISDIR(st.st_mode))
		{
			if (!GetFileList(sPath.c_str()))
			{
				bResult = false;
				break;
			}
		}
		else if (S_ISREG(st.st_mode))
		{
			if(sPath.length() > 5)
			{
				String ext = sPath.substr(sPath.length() - 4);
				if (ext == ".jpg" || ext == ".JPG" || ext == ".png" || ext == ".PNG")
					m_List.push_back(sPath);
			}
		}
	}

	closedir(pDir);
	return bResult;
}
