/****************************************************************************

 ■PhotoFrame
	設定ファイルアクセス
        Copyright(C) 2018 Denshi Usagi

****************************************************************************/

#include "Settings.h"
#include "SDL2/SDL.h"
#include "main.h"
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>

// 設定ファイルの書式
// {Name} = {Value}
// "#"以降はコメント
// 空白やタブ文字は無視
// 改行コードは LF or CRLF 両方OK

//---------------------------------------------------------------------------

// ●コンストラクタ
CSettings::CSettings(PCSTR pFile)
{
	m_FilePath = pFile;
	m_TempPath = "_" + m_FilePath;
}

// ●文字列のリード
String CSettings::ReadStr(PCSTR pKey, PCSTR pDefault)
{
	String sRet = pDefault;

	FILE* fp = fopen(m_FilePath.c_str(), "r");
	if (fp != NULL)
	{
		char buf[1024];
		while (fgets(buf, sizeof(buf), fp) != NULL)
		{
			PSTR p = strchr(buf, '#');
			if (p != NULL) *p = '\0';
			p = strchr(buf, '\r');
			if (p != NULL) *p = '\0';
			p = strchr(buf, '\n');
			if (p != NULL) *p = '\0';

			p = strchr(buf, '=');
			if (p != NULL)
			{
				*p = '\0';
				String strKey = Trim(String(buf));
				if (strKey == pKey)
				{
					sRet = Trim(String(p + 1));
					break;
				}
			}
		}
		fclose(fp);
	}

	return sRet;
}

// ●数値のリード
int CSettings::ReadInt(PCSTR pKey, int nDefault)
{
	int nRet = nDefault;

	String strVal = ReadStr(pKey, "");
	if (strVal != "")
		nRet = atoi(strVal.c_str());

	return nRet;
}

// ●文字列のライト
void CSettings::WriteStr(PCSTR pKey, PCSTR pValue)
{
	FILE* fp = fopen(m_FilePath.c_str(), "a+");
	FILE* wp = fopen(m_TempPath.c_str(), "w");

	if (fp == NULL || wp == NULL)
	{
		MessageBox("CSettings", "設定ファイルのオープンに失敗");
		if (fp != NULL) fclose(fp);
		if (wp != NULL) fclose(wp);
		return;
	}

	bool ferr = false;
	bool found = false;
	String strNew = pKey;
	strNew += " = ";
	strNew += pValue;

	char buf[512];
	while (fgets(buf, sizeof(buf), fp) != NULL)
	{
		String strLine = buf;
		strLine = Trim(strLine);

		PSTR p = strchr(buf, '#');
		if (p != NULL) *p = '\0';
		p = strchr(buf, '\r');
		if (p != NULL) *p = '\0';
		p = strchr(buf, '\n');
		if (p != NULL) *p = '\0';

		p = strchr(buf, '=');
		if (p != NULL)
		{
			*p = '\0';
			String strKey = Trim(String(buf));
			if (strKey == pKey)
			{
				strLine = strNew;
				found = true;
			}
		}

		strLine += "\r\n";
		if (fputs(strLine.c_str(), wp) == EOF)
		{
			ferr = true;
			break;
		}
	}
	if (!ferr && !found)
	{
		strNew += "\r\n";
		if (fputs(strNew.c_str(), wp) == EOF)
			ferr = true;
	}

	fclose(fp);
	fclose(wp);

	if (!ferr)
	{
	    struct stat st;
	    bool exist = (stat(m_FilePath.c_str(), &st) == 0);

		if (!exist || remove(m_FilePath.c_str()) == 0)
		{
			if (rename(m_TempPath.c_str(), m_FilePath.c_str()) == 0)
				return;
		}
	}

	MessageBox("CSettings", "設定ファイルの書き込みに失敗");
}

// ●数値のライト
void CSettings::WriteInt(PCSTR pKey, int nValue)
{
	char buf[32];
	snprintf(buf, sizeof(buf), "%d", nValue);
	WriteStr(pKey, buf);
}

// ●文字列のTrim
String CSettings::Trim(const String& str)
{
	String sRet;

	#define TRIM_CHARS	 " \t\v\r\n"

	int left = str.find_first_not_of(TRIM_CHARS);
	if (left == std::string::npos)
		return sRet;

	int right = str.find_last_not_of(TRIM_CHARS);

	if (left <= right)
		sRet = str.substr(left, right - left + 1);

	return sRet;
}
