/****************************************************************************

 ■PhotoFrame
	天気情報
        Copyright(C) 2018 Denshi Usagi

****************************************************************************/

#include "Weather.h"
#include "main.h"
#include <stdlib.h>
#include <string.h>

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include "SDL.h"

//---------------------------------------------------------------------------

// ●コンストラクタ
CWeather::CWeather()
{
	m_bEnable = false;
	memset(&m_Weather, 0, sizeof(m_Weather));
	m_Socket = -1;
	m_pCtx = NULL;
	m_pSSL = NULL;
	m_tLast = 0;
	m_pBuf1 = NULL;
	m_pBuf2 = NULL;
}

// ●初期化
void CWeather::Initialize(void)
{
	// 天気取得元の情報
	m_strHost = g_Settings.ReadStr("WeatherHost", "");
	m_strQuery1 = g_Settings.ReadStr("WeatherQuery1", ""); // 1時間天気用
	m_strQuery2 = g_Settings.ReadStr("WeatherQuery2", ""); // 週間天気用
}

// ●ネットから天気情報を取得する
void CWeather::Update(void)
{
	// 取得元が設定されていない場合は天気表示不可
	if (m_strHost == "" || m_strQuery1 == "" || m_strQuery2 == "")
		return;

	time_t tCurrent = time(NULL);
	struct tm tmCur = *localtime(&tCurrent);

	// 取得処理(サーバーアクセス)の間引き
	//m_tLast = 0; // for Debug
	if (m_tLast > 0)
	{
		if (m_bEnable)
		{
			// 正常に取得できている場合

			// 取得済みの時間が最新になるまで10秒おきにリトライする(5分に1回)
			int nHour = (tmCur.tm_hour + 1) % 24;
			
			if (m_Weather.Detail[0].nHour == nHour && (tCurrent - m_tLast) < (5 * 60))
				return;
			if ((tCurrent - m_tLast) < 10)
				return;
		}
		else
		{
			// 正常に取得できていない場合は1分おきにリトライする
			// URLの変更やHTMLの仕様が変わった可能性もある
			if ((tCurrent - m_tLast) < (1 * 60))
				return;
		}
	}

	// 接続から天気情報取得まで
	char buf[128], tbf[64];
	strftime(tbf, sizeof(tbf), "%Y/%m/%d %H:%M:%S", &tmCur);

	int nErr = Request(m_strQuery1.c_str(), m_pBuf1, 200 * 1024); // 1時間天気用
	if (nErr == 0)
	{
		nErr = Request(m_strQuery2.c_str(), m_pBuf2, 200 * 1024); // 週間天気用
		if (nErr == 0)
		{
			nErr = ParseHtml();
			if (nErr != 0)
			{
				snprintf(buf, sizeof(buf), "天気情報の取得に失敗(nErr = %d) %s", nErr, tbf);
				g_SaveData.WriteStr("LastError", buf);
			}
		}
		else
		{
			snprintf(buf, sizeof(buf), "天気情報のリクエストに失敗_2(nErr = %d) %s", nErr, tbf);
			g_SaveData.WriteStr("LastError", buf);
		}
	}
	else
	{
		snprintf(buf, sizeof(buf), "天気情報のリクエストに失敗_1(nErr = %d) %s", nErr, tbf);
		g_SaveData.WriteStr("LastError", buf);
	}
	
	// リクエスト終了
	EndRequest();

	m_tLast = tCurrent;
}

// ●天気情報が有効かどうか
bool CWeather::IsEnable(void)
{
	return m_bEnable;
}

// ●現在の天気情報を取得する
const SWeather* CWeather::GetWeather(void)
{
	return &m_Weather;
}

 // ●HTTPリクエストを送信してHTMLを受信する
int CWeather::Request(PCSTR pQuery, PSTR& pBuf, int nSize)
{
	// バッファ確保
	pBuf = (char*)malloc(nSize + 1);
	if (pBuf == NULL)
		return 1;

	memset(pBuf, 0 , nSize + 1);

	// ホスト解決
	addrinfo hints;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;
	
	addrinfo* res;
    if (getaddrinfo(m_strHost.c_str(), "https", &hints, &res) != 0)
        return 2;
	
	addrinfo info = *res;
	freeaddrinfo(res);

	// ソケット作成
	m_Socket = socket(info.ai_family, info.ai_socktype, info.ai_protocol);
	if (m_Socket == -1)
		return 3;
	
	// ソケット接続
	if (connect(m_Socket, info.ai_addr, info.ai_addrlen) != 0)
		return 4;
 
	// SSL初期化
    m_pCtx = SSL_CTX_new(SSLv23_client_method());
	if (m_pCtx == NULL)
		return 5;
		
	SSL_CTX_set_mode(m_pCtx, SSL_MODE_AUTO_RETRY);

	if (SSL_CTX_set_min_proto_version(m_pCtx, TLS1_2_VERSION) == 0)
		return 6;

	if (SSL_CTX_set_default_verify_paths(m_pCtx) == 0)
		return 7;
    
    m_pSSL = SSL_new(m_pCtx);
	if (m_pSSL == NULL)
		return 8;

    if (SSL_set_tlsext_host_name(m_pSSL, m_strHost.c_str()) == 0)
		return 9;

	if (SSL_set_fd(m_pSSL, m_Socket) == 0)
		return 10;
			
    // SSL接続
	if (SSL_connect(m_pSSL) != 1)
		return 11;
 
	// ヘッダ送信	
	String strRequest;
	strRequest.append("GET ");
	strRequest.append(pQuery);
	strRequest.append(" HTTP/1.1\r\nHost: ");
	strRequest.append(m_strHost.c_str());
	strRequest.append("\r\nConnection: close");
	strRequest.append("\r\n\r\n");

	int nSend = SSL_write(m_pSSL, strRequest.c_str(), strRequest.size());
	if (nSend != strRequest.size())
		return 12;

	// レスポンス受信
	int nRecv = SSL_read(m_pSSL, pBuf, 17);
	if (nRecv != 17 || strstr(pBuf, "HTTP/1.1 200 OK\r\n") != pBuf)
		return 13;

	// 受信処理
	PSTR pRecv = pBuf;
	while(nSize > 0)
	{
		SDL_Delay(100);

		nRecv = SSL_read(m_pSSL, pRecv, nSize);
		if (nRecv <= 0)
			break;
		
		pRecv += nRecv;
		nSize -= nRecv;
	}
	if (pRecv == pBuf)
		return 14;
	
#if 0 // デバッグ用(受信したHTML文書をカレントディレクトリに吐き出す)
	PCSTR pName = (m_strQuery1 == pQuery)? "Query1-recv.txt" : "Query2-recv.txt";
	FILE* fp = fopen(pName, "w+");
	if (fp)
	{
		fputs(pBuf, fp);
		fclose(fp);
	}
#endif

	return 0;
}

// ●HTMLを受信して天気情報を取得する
int CWeather::ParseHtml(void)
{
	m_bEnable = false;

	PSTR pGet1 = m_pBuf1 + 4 * 1024; // 先頭読み飛ばし
	//RemoveStr(m_pBuf1, pGet1, "\r\n2000\r\n"); // 妙な4桁文字列が紛れているので
	pGet1 = m_pBuf1;

	if (!Search(pGet1, "wTable time"))
		return 1;
	if (!Search(pGet1, "wTable__content"))
		return 1;

	// 天気情報取得
	SWeather data;
	memset(&data, 0, sizeof(data));

	for (int i = 0; i < 24 ; i++) // 1時間ごとのピンポイント天気
	{
		// 時間
		if (!GetValue(pGet1, "\"wTable__item time\">", "<", data.Detail[i].nHour))
			return 2;

		// 天気(アイコン)
		if (!GetValue(pGet1, "/img/wxicon/", ".png", data.Detail[i].nTenki))
			return 3;

		// 降水量
		if (!GetValue(pGet1, "\"wTable__item r\">", "<", data.Detail[i].nKosui))
			return 4;

		// 気温
		if (!GetValue(pGet1, "\"wTable__item t\">", "<", data.Detail[i].nKion))
			return 5;
	}

	PSTR pGet2 = m_pBuf2 + 4 * 1024; // 先頭読み飛ばし
	//RemoveStr(m_pBuf2, pGet2, "\r\n2000\r\n"); // 妙な4桁文字列が紛れているので
	pGet2 = m_pBuf2;

	if (!Search(pGet2, "wTable week"))
		return 6;
	if (!Search(pGet2, "wTable__content"))
		return 6;

	for (int i = 0; i < 8 ; i++) // 週間天気(最大10個あるが表示は8個)
	{
		// 日
		if (!GetValue(pGet2, "\"wTable__item day\">", "<", data.Weekly[i].nDay))
			return 7;

		// 天気(アイコン)
		if (!GetValue(pGet2, "/img/wxicon/", ".png", data.Weekly[i].nTenki))
			return 8;

		// 最高気温
		if (!GetValue(pGet2, "\"wTable__item h txt-h\">", "<", data.Weekly[i].nKion))
			return 9;

		// 降水確率
		if (!GetValue(pGet2, "\"wTable__item r\">", "<", data.Weekly[i].nKosui))
			return 10;
	}

	// 取得成功
	m_Weather = data;
	m_bEnable = true;

	return 0;
}

// ●データ位置のサーチ
bool CWeather::Search(PSTR& pBuf, PCSTR pKey)
{
	PSTR pTop = strstr(pBuf, pKey);
	if (pTop != NULL)
	{
		pBuf = pTop + strlen(pKey);
		return true;
	}
	return false;
}

// ●データ取得位置の検索
bool CWeather::GetValue(PSTR& pBuf, PCSTR pKey1, PCSTR pKey2, int& var)
{
	PSTR pTop = strstr(pBuf, pKey1);
	if (pTop == NULL)
		return false;

	pTop += strlen(pKey1);
	PSTR pEnd = strstr(pTop, pKey2);
	if (pEnd == NULL)
		return false;

	pBuf = pEnd + strlen(pKey2);
	*pEnd = '\0';

	var = atoi(pTop);
	return true;
}

// ●リクエストを終了する
void CWeather::EndRequest(void)
{
	if (m_pSSL != NULL)
	{
		SSL_shutdown(m_pSSL);
		SSL_free(m_pSSL);
		m_pSSL = NULL;
	}
	if (m_pCtx != NULL)
	{
		SSL_CTX_free(m_pCtx);
		m_pCtx = NULL;
	}
	if (m_Socket != -1)
	{
		close(m_Socket);
		m_Socket = -1;
	}
    
	if (m_pBuf1 != NULL)
	{
		free(m_pBuf1);
		m_pBuf1 = NULL;
	}
	if (m_pBuf2 != NULL)
	{
		free(m_pBuf2);
		m_pBuf2 = NULL;
	}
}

// ●文字列から文字列を削除
void CWeather::RemoveStr(PSTR pDst, PSTR pSrc, PCSTR pKey)
{
	*pDst = '\0';

	PSTR pCur = pSrc;
	int nLen = strlen(pKey);

	for(;;)
	{
		PSTR p = strstr(pCur, pKey);
		if (p == NULL)
		{
			strcat(pDst, pCur);
			return;
		}
		*p = '\0';
		strcat(pDst, pCur);
		pCur = p + nLen;
	}
}
