/****************************************************************************

 ■PhotoFrame
	メイン
		Copyright(C) 2018 Denshi Usagi

****************************************************************************/

#include "main.h"
#include "Application.h"
#include "SDL2/SDL.h"
#include "SDL_image.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"
#include "SDL_net.h"
#include <openssl/ssl.h>
#include <unistd.h>

#define MUSIC_CMD	"cvlc --play-and-exit --no-interact"

static bool Initialize(void);
static bool Run(void);
static void Finalize(bool fAppExit);

CCtrlBoard g_CtrlBoard;
CSettings g_Settings("Setting.ini");
CSettings g_SaveData("SaveData.ini");

static CApplication s_App;

//---------------------------------------------------------------------------

// ●メイン
int main(int argc, char *argv[])
{
	// startupで起動した場合
	if (argc >= 2)
	{
		// カレントディレクトリを設定する
		chdir(argv[1]);

		// OS起動直後なので安定するまで待つ
		SDL_Delay(20 * 1000);
	}

	bool fAppExit = true;

	// 初期化
	if (Initialize())
	{
		// 実行
		fAppExit = Run();
	}
	// 終了
	Finalize(fAppExit);

	return 0;
}

// ●初期化
static bool Initialize(void)
{
	// SDLの初期化
	if (SDL_Init(SDL_INIT_EVERYTHING) != 0)
		return false; // リモートデバッグではエラーになるのでアタッチする

	// SDL_imageの初期化
	int flags = IMG_INIT_JPG | IMG_INIT_PNG;
	if ((IMG_Init(flags) & flags) != flags)
		return false;

	// SDL_fontの初期化
	if (TTF_Init() != 0)
		return false;

	// SDL_mixerの初期化
	if ((Mix_Init(MIX_INIT_MP3) & MIX_INIT_MP3) != MIX_INIT_MP3)
		return false;

	if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 2, 4096) != 0)
		return false;

	if (Mix_SetMusicCMD(MUSIC_CMD) != 0)
		return false;

	// SDL_netの初期化
	if (SDLNet_Init() != 0)
		return false;

	if (!g_CtrlBoard.Initialize())
		return false;

	if (!s_App.Initialize())
		return false;
	
	// OpenSSLの初期化
    SSL_library_init();

	return true;
}

// ●実行
static bool Run(void)
{
	for(;;)
	{
		// フォトフレーム処理
		switch (s_App.Process())
		{
		case 1: // ユーザー操作による終了
			return true;

		case 2: // 通常終了(スケジュールなし)
			return false;
		}
		
		SDL_Delay(1000);
	}
}

// ●終了
static void Finalize(bool fAppExit)
{
	s_App.Finalize();

	SDLNet_Quit();
	Mix_CloseAudio();
	Mix_Quit();
	TTF_Quit();
	IMG_Quit();
	SDL_Quit();

	// 非常終了では以降の電断準備は行わない
	if (fAppExit)
		return;

	// 終了通知
	// この後、一定時間が経過すると電源がOFFされる
	g_CtrlBoard.NotifyEixt();

	//シャットダウン
	system("shutdown -h now");
}

// ●エラーメッセージボックス
void MessageBox(PCSTR pTitle, PCSTR pMsg)
{
	g_SaveData.WriteStr("LastError", pMsg);

	g_CtrlBoard.EnablePanel(true);
	SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, pTitle, pMsg, NULL);
}
